<?php

namespace Bixo\Contact\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Bixo\Contact\Http\Requests\ContactsRequest;
use Bixo\Contact\Interfaces\ContactsRepositoryInterface;
use Bixo\Contact\Models\Contacts;

/**
 * Resource controller class for contacts.
 */
class ContactsResourceController extends BaseController
{

    /**
     * Initialize contacts resource controller.
     *
     * @param type ContactsRepositoryInterface $contacts
     *
     * @return null
     */
    public function __construct(ContactsRepositoryInterface $contacts)
    {
        parent::__construct();
        $this->repository = $contacts;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Bixo\Contact\Repositories\Criteria\ContactsResourceCriteria::class);
    }

    /**
     * Display a list of contacts.
     *
     * @return Response
     */
    public function index(ContactsRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Bixo\Contact\Repositories\Presenter\ContactsPresenter::class)
                ->$function();
        }

        $contacts = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('contact::contacts.names'))
            ->view('contact::contacts.index', true)
            ->data(compact('contacts', 'view'))
            ->output();
    }

    /**
     * Display contacts.
     *
     * @param Request $request
     * @param Model   $contacts
     *
     * @return Response
     */
    public function show(ContactsRequest $request, Contacts $contacts)
    {

        if ($contacts->exists) {
            $view = 'contact::contacts.show';
        } else {
            $view = 'contact::contacts.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('contact::contacts.name'))
            ->data(compact('contacts'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new contacts.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(ContactsRequest $request)
    {

        $contacts = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('contact::contacts.name')) 
            ->view('contact::contacts.create', true) 
            ->data(compact('contacts'))
            ->output();
    }

    /**
     * Create new contacts.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(ContactsRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $contacts                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('contact::contacts.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('contact/contacts/' . $contacts->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/contact/contacts'))
                ->redirect();
        }

    }

    /**
     * Show contacts for editing.
     *
     * @param Request $request
     * @param Model   $contacts
     *
     * @return Response
     */
    public function edit(ContactsRequest $request, Contacts $contacts)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('contact::contacts.name'))
            ->view('contact::contacts.edit', true)
            ->data(compact('contacts'))
            ->output();
    }

    /**
     * Update the contacts.
     *
     * @param Request $request
     * @param Model   $contacts
     *
     * @return Response
     */
    public function update(ContactsRequest $request, Contacts $contacts)
    {
        try {
            $attributes = $request->all();

            $contacts->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('contact::contacts.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('contact/contacts/' . $contacts->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('contact/contacts/' . $contacts->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the contacts.
     *
     * @param Model   $contacts
     *
     * @return Response
     */
    public function destroy(ContactsRequest $request, Contacts $contacts)
    {
        try {

            $contacts->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('contact::contacts.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('contact/contacts/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('contact/contacts/' . $contacts->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple contacts.
     *
     * @param Model   $contacts
     *
     * @return Response
     */
    public function delete(ContactsRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('contact::contacts.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('contact/contacts'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/contact/contacts'))
                ->redirect();
        }

    }

    /**
     * Restore deleted contacts.
     *
     * @param Model   $contacts
     *
     * @return Response
     */
    public function restore(ContactsRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('contact::contacts.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/contact/contacts'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/contact/contacts/'))
                ->redirect();
        }

    }

}
